@@ -11,8 +11,9 @@ import sys
 import time
 import warnings
 import decimal
+
 import jpype
-from .sqloption import SQLOptions
+from datetime import date
 
 
 class SQLCliJDBCException(Exception):
@@ -24,6 +25,10 @@ class SQLCliJDBCException(Exception):
         return self.message
 
 
+class SQLCliJDBCTimeOutException(SQLCliJDBCException):
+    pass
+
+
 def reraise(tp, value, tb=None):
     if value is None:
         value = tp()
@@ -44,8 +49,6 @@ _jdbc_name_to_const = None
 # Mapping from java.sql.Types attribute constant value to it's attribute name
 _jdbc_const_to_name = []
 
-_sqloptions = SQLOptions()
-
 # 默认情况下blob字段内容输出的最大长度
 _blobdefaultfetchsize = 20
 # 默认情况下clob字段内容输出的最大长度
@@ -66,55 +69,24 @@ def _handle_sql_exception_jpype():
     reraise(exc_type, exc_info[1], exc_info[2])
 
 
-def _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs):
-    import jpype
-    if not getattr(jpype, "isJVMStarted")():
-        args = []
-        class_path = []
-        if jars:
-            class_path.extend(jars)
-        class_path.extend(_get_classpath())
-        if class_path:
-            args.append('-Djava.class.path=%s' %
-                        os.path.pathsep.join(class_path))
-        if libs:
-            # path to shared libraries
-            libs_path = os.path.pathsep.join(libs)
-            args.append('-Djava.library.path=%s' % libs_path)
-        jvm_path = getattr(jpype, "getDefaultJVMPath")()
-        getattr(jpype, "startJVM")(jvm_path, *args, ignoreUnrecognized=True, convertStrings=True)
-    if not getattr(jpype, "isThreadAttachedToJVM")():
-        getattr(jpype, "attachThreadToJVM")()
-        jpype.java.lang.Thread.currentThread().setContextClassLoader(jpype.java.lang.ClassLoader.getSystemClassLoader())
-    if _jdbc_name_to_const is None:
-        types = jpype.java.sql.Types
-        types_map = {}
-        for i in types.class_.getFields():
-            if jpype.java.lang.reflect.Modifier.isStatic(i.getModifiers()):
-                const = i.get(None)
-                types_map[i.getName()] = const
-        _init_types(types_map)
-
-    # register driver for DriverManager
-    try:
-        jpype.JClass(jclassname)
-    except TypeError:
-        raise SQLCliJDBCException("SQLCLI-00000: Load java class failed. [" + str(jclassname) + "]")
-    if isinstance(driver_args, dict):
-        Properties = jpype.java.util.Properties
-        info = Properties()
-        for k, v in driver_args.items():
-            info.setProperty(k, v)
-        dargs = [info]
-    else:
-        dargs = driver_args
-
-    try:
-        return jpype.java.sql.DriverManager.getConnection(url, *dargs)
-    except jpype.java.sql.SQLException as je:
-        raise SQLCliJDBCException(je.toString().replace("java.sql.SQLException: ", "").
-                                  replace("java.sql.SQLTransientConnectionException: ", "").
-                                  replace("java.sql.SQLInvalidAuthorizationSpecException: ", ""))
+def limit(method, timeout):
+    """ Convert a Java method to asynchronous call with a specified timeout. """
+    """ 这里不使用func_timeout， 是因为funct_timeout的函数内部如果启动新的线程，会导致Python进程最后无法退出的问题"""
+    def f(*args):
+        @jpype.JImplements(jpype.java.util.concurrent.Callable)
+        class g:
+            @jpype.JOverride
+            def call(self):
+                return method(*args)
+        future = jpype.java.util.concurrent.FutureTask(g())
+        jpype.java.lang.Thread(future).start()
+        try:
+            timeunit = jpype.java.util.concurrent.TimeUnit.SECONDS
+            return future.get(int(timeout), timeunit)
+        except jpype.java.util.concurrent.TimeoutException:
+            future.cancel(True)
+            raise SQLCliJDBCTimeOutException("TimeOut")
+    return f
 
 
 def _get_classpath():
@@ -274,8 +246,7 @@ def setClobDefaultFetchSize(p_nClobDefaultFetchSize):
     _clobdefaultfetchsize = p_nClobDefaultFetchSize
 
 
-# DB-API 2.0 Module Interface connect constructor
-def connect(jclassname, url, driver_args=None, jars=None, libs=None):
+def AttachJVM(jars=None, libs=None):
     """Open a connection to a database using a JDBC driver and return
     a Connection instance.
 
@@ -292,10 +263,6 @@ def connect(jclassname, url, driver_args=None, jars=None, libs=None):
     libs: Dll/so filenames or sequence of dlls/sos used as shared
           library by the JDBC driver
     """
-    if isinstance(driver_args, string_type):
-        driver_args = [driver_args]
-    if not driver_args:
-        driver_args = []
     if jars:
         if isinstance(jars, string_type):
             jars = [jars]
@@ -307,8 +274,98 @@ def connect(jclassname, url, driver_args=None, jars=None, libs=None):
     else:
         libs = []
 
-    # 如果无法连接，则尝试3次，间隔等待2秒
-    jconn = _jdbc_connect_jpype(jclassname, url, driver_args, jars, libs)
+    import jpype
+    if not getattr(jpype, "isJVMStarted")():
+        args = []
+        class_path = []
+        if jars:
+            class_path.extend(jars)
+        class_path.extend(_get_classpath())
+        if class_path:
+            args.append('-Djava.class.path=%s' %
+                        os.path.pathsep.join(class_path))
+        if libs:
+            # path to shared libraries
+            libs_path = os.path.pathsep.join(libs)
+            args.append('-Djava.library.path=%s' % libs_path)
+        jvm_path = getattr(jpype, "getDefaultJVMPath")()
+        getattr(jpype, "startJVM")(jvm_path, *args, ignoreUnrecognized=True, convertStrings=True)
+
+    if not getattr(jpype, "isThreadAttachedToJVM")():
+        getattr(jpype, "attachThreadToJVM")()
+        jpype.java.lang.Thread.currentThread().setContextClassLoader(jpype.java.lang.ClassLoader.getSystemClassLoader())
+    if _jdbc_name_to_const is None:
+        types = jpype.java.sql.Types
+        types_map = {}
+        for i in types.class_.getFields():
+            if jpype.java.lang.reflect.Modifier.isStatic(i.getModifiers()):
+                const = i.get(None)
+                types_map[i.getName()] = const
+        _init_types(types_map)
+
+
+def DetachJVM():
+    try:
+        if getattr(jpype, "isThreadAttachedToJVM")():
+            jpype.java.lang.Thread.detach()
+    except ImportError:
+        # JPype可能已经被卸载，不再重复操作
+        pass
+
+
+# DB-API 2.0 Module Interface connect constructor
+def connect(jclassname, url, driver_args=None, jars=None, libs=None, TimeOutLimit=-1):
+    """Open a connection to a database using a JDBC driver and return
+    a Connection instance.
+
+    jclassname: Full qualified Java class name of the JDBC driver.
+    url: Database url as required by the JDBC driver.
+    driver_args: Dictionary or sequence of arguments to be passed to
+           the Java DriverManager.getConnection method. Usually
+           sequence of username and password for the db. Alternatively
+           a dictionary of connection arguments (where `user` and
+           `password` would probably be included). See
+           http://docs.oracle.com/javase/7/docs/api/java/sql/DriverManager.html
+           for more details
+    jars: Jar filename or sequence of filenames for the JDBC driver
+    libs: Dll/so filenames or sequence of dlls/sos used as shared
+          library by the JDBC driver
+    """
+    # 连接到JVM上
+    AttachJVM(jars=jars, libs=libs)
+
+    if isinstance(driver_args, string_type):
+        driver_args = [driver_args]
+    if not driver_args:
+        driver_args = []
+
+    # register driver for DriverManager
+    try:
+        jpype.JClass(jclassname)
+    except TypeError:
+        raise SQLCliJDBCException("SQLCLI-00000: Load java class failed. [" + str(jclassname) + "]")
+    if isinstance(driver_args, dict):
+        Properties = jpype.java.util.Properties
+        info = Properties()
+        for k, v in driver_args.items():
+            info.setProperty(k, v)
+        dargs = [info]
+    else:
+        dargs = driver_args
+
+    try:
+        if TimeOutLimit != -1:
+            connect_jdbc = limit(jpype.java.sql.DriverManager.getConnection, TimeOutLimit)
+        else:
+            connect_jdbc = jpype.java.sql.DriverManager.getConnection
+        jconn = connect_jdbc(url, *dargs)
+    except SQLCliJDBCTimeOutException:
+        raise SQLCliJDBCException("SQLCLI-0000:: Timeout expired. Abort this command.")
+    except jpype.java.sql.SQLException as je:
+        raise SQLCliJDBCException(je.toString().replace("java.sql.SQLException: ", "").
+                                  replace("java.sql.SQLTransientConnectionException: ", "").
+                                  replace("java.sql.SQLInvalidAuthorizationSpecException: ", ""))
+
     return Connection(jconn, _converters)
 
 
@@ -461,14 +518,18 @@ class Cursor(object):
             else:
                 prep_stmt.setObject(i + 1, parameters[i])
 
-    def execute_direct(self, operation):
+    def execute_direct(self, operation, TimeOutLimit=-1):
         if self._connection.isClosed():
             raise Error()
         self._close_last()
         is_rs = False
         self._stmt = self._connection.jconn.createStatement()
         try:
-            is_rs = self._stmt.execute(operation)
+            if TimeOutLimit != -1:
+                execute = limit(self._stmt.execute, TimeOutLimit)
+            else:
+                execute = self._stmt.execute
+            is_rs = execute(operation)
             m_SQLWarnMessage = None
             m_SQLWarnings = self._stmt.getWarnings()
             while m_SQLWarnings is not None:
@@ -478,6 +539,8 @@ class Cursor(object):
                     m_SQLWarnMessage = m_SQLWarnMessage + "\n" + m_SQLWarnings.getMessage()
                 m_SQLWarnings = m_SQLWarnings.getNextWarning()
             self.warnings = m_SQLWarnMessage
+        except SQLCliJDBCTimeOutException:
+            raise SQLCliJDBCTimeOutException("SQLCLI-0000:: Timeout expired. Abort this command.")
         except Exception:
             _handle_sql_exception_jpype()
         self._rs = self._stmt.getResultSet()
@@ -492,7 +555,7 @@ class Cursor(object):
         else:
             self.rowcount = self._stmt.getUpdateCount()
 
-    def execute(self, operation, parameters=None):
+    def execute(self, operation, parameters=None, TimeOutLimit=-1):
         if self._connection.isClosed():
             raise Error()
         if not parameters:
@@ -502,7 +565,11 @@ class Cursor(object):
         self._set_stmt_parms(self._prep, parameters)
         is_rs = False
         try:
-            is_rs = self._prep.execute()
+            if TimeOutLimit != -1:
+                execute = limit(self._prep.execute, TimeOutLimit)
+            else:
+                execute = self._prep.execute
+            is_rs = execute()
             m_SQLWarnMessage = None
             m_SQLWarnings = self._prep.getWarnings()
             while m_SQLWarnings is not None:
@@ -512,6 +579,8 @@ class Cursor(object):
                     m_SQLWarnMessage = m_SQLWarnMessage + "\n" + m_SQLWarnings.getMessage()
                 m_SQLWarnings = m_SQLWarnings.getNextWarning()
             self.warnings = m_SQLWarnMessage
+        except SQLCliJDBCTimeOutException:
+            raise SQLCliJDBCTimeOutException("SQLCLI-0000:: Timeout expired. Abort this command.")
         except Exception:
             _handle_sql_exception_jpype()
         # 忽略对execute的返回判断，总是恒定的去调用getResultSet
@@ -639,15 +708,20 @@ def _to_datetime(conn, rs, col):
     if conn:
         pass
     java_val = rs.getTimestamp(col)
+
     if not java_val:
         return
     try:
-        d = datetime.datetime.strptime(str(java_val)[:19], "%Y-%m-%d %H:%M:%S")
-        d = d.replace(microsecond=int(str(java_val.getNanos())[:6]))
+        ld = java_val.toLocalDateTime()
+        d = datetime.datetime.strptime(str(ld.getYear()) + "-" + str(ld.getMonthValue()) + "-" +
+                                       str(ld.getDayOfMonth()) + " " +
+                                       str(ld.getHour()) + ":" + str(ld.getMinute()) + ":" + str(ld.getSecond()) +
+                                       " " + str(ld.getNano()), "%Y-%m-%d %H:%M:%S %f")
+        return d
     except ValueError:
         # 处理一些超过Python数据类型限制的时间类型
         d = str(java_val)
-    return str(d)
+    return d
 
 
 def _to_time(conn, rs, col):
@@ -688,12 +762,9 @@ def _to_date(conn, rs, col):
         return rs.getString(col)
     if not java_val:
         return
-    # The following code requires Python 3.3+ on dates before year 1900.
-    # d = datetime.datetime.strptime(str(java_val)[:10], "%Y-%m-%d")
-    # return d.strftime("%Y-%m-%d")
-    # Workaround / simpler soltution (see
-    # https://github.com/baztian/jaydebeapi/issues/18):
-    return str(java_val)[:10]
+    d = date.fromisocalendar(java_val.toLocalDate().getYear(), java_val.toLocalDate().getMonthValue(),
+                             java_val.toLocalDate().getDayOfMonth())
+    return d
 
 
 def _to_bit(conn, rs, col):
@@ -972,8 +1043,8 @@ _DEFAULT_CONVERTERS = {
     'YEAR':                         _to_year,
     'VARBINARY':                    _to_varbinary,
     'BINARY':                       _to_binary,
-    'LONGVARBINARY':                _to_binary,
-    'BLOB':                         _to_binary,
+    'LONGVARBINARY':                _to_varbinary,
+    'BLOB':                         _to_varbinary,
     'BFILE':                        _to_binary,
     'DECIMAL':                      _java_to_py_bigdecimal,
     'NUMERIC':                      _java_to_py_bigdecimal,
