@@ -8,8 +8,6 @@ import re
 import traceback
 import json
 import binascii
-from func_timeout import func_set_timeout
-import func_timeout
 
 from .sqlparse import SQLAnalyze
 from .sqlparse import SQLFormatWithPrefix
@@ -17,6 +15,7 @@ from .commandanalyze import execute
 from .commandanalyze import CommandNotFound
 from .sqlcliexception import SQLCliException
 from .sqlclijdbcapi import SQLCliJDBCException
+from .sqlclijdbcapi import SQLCliJDBCTimeOutException
 from .sqlcliodbc import SQLCliODBCException
 
 
@@ -273,8 +272,11 @@ class SQLExecute(object):
                 # 处理超时时间问题
                 if m_ScriptTimeOut > 0:
                     if m_ScriptTimeOut <= time.time() - self.getStartTime():
-                        m_SQL_ErrorMessage = "SQLCLI-0000: Script imeout when execute script. abort it."
-                        yield {"type": "error", "message": m_SQL_ErrorMessage}
+                        if sql.upper() not in ["EXIT", "QUIT"]:
+                            m_SQL_ErrorMessage = "SQLCLI-0000: Script Timeout " \
+                                                 "(" + str(round(time.time() - self.getStartTime(), 2)) + \
+                                                 ") expired. Abort this Script."
+                            yield {"type": "error", "message": m_SQL_ErrorMessage}
                         raise EOFError
                     else:
                         if m_SQLTimeOut <= 0 or (m_ScriptTimeOut - (time.time() - self.getStartTime()) < m_SQLTimeOut):
@@ -350,6 +352,19 @@ class SQLExecute(object):
                             m_Result["rows"] = []
                             m_Result["title"] = ""
                     yield m_Result
+            except SQLCliJDBCTimeOutException:
+                # 处理超时时间问题
+                if m_ScriptTimeOut > 0:
+                    if m_ScriptTimeOut <= time.time() - self.getStartTime():
+                        if sql.upper() not in ["EXIT", "QUIT"]:
+                            m_SQL_ErrorMessage = "SQLCLI-0000: Script Timeout " \
+                                                 "(" + str(round(time.time() - self.getStartTime(), 2)) + \
+                                                 ") expired. Abort this Script."
+                            yield {"type": "error", "message": m_SQL_ErrorMessage}
+                        raise EOFError
+                m_SQL_ErrorMessage = "SQLCLI-0000: SQL Timeout (" + str(time.time() - self.getStartTime()) + \
+                                     ") expired. Abort this command."
+                yield {"type": "error", "message": m_SQL_ErrorMessage}
             except CommandNotFound:
                 # 进入到SQL执行阶段, 开始执行SQL语句
                 if self.conn:
@@ -371,35 +386,29 @@ class SQLExecute(object):
                     try:
                         if "SQLCLI_DEBUG" in os.environ:
                             print("DEBUG-SQL=[" + str(sql) + "]")
-
-                        # 执行SQL脚本
-                        def executesql():
-                            if "SQL_DIRECT" in m_SQLHint.keys():
-                                self.cur.execute_direct(sql)
-                            elif "SQL_PREPARE" in m_SQLHint.keys():
-                                self.cur.execute(sql)
-                            else:
-                                if self.SQLOptions.get("SQL_EXECUTE").upper() == "DIRECT":
-                                    self.cur.execute_direct(sql)
-                                else:
-                                    self.cur.execute(sql)
-
                         # 处理超时时间问题
                         if m_ScriptTimeOut > 0:
                             if m_ScriptTimeOut <= time.time() - self.getStartTime():
-                                m_SQL_ErrorMessage = "SQLCLI-0000: Script imeout when execute script. abort it."
-                                yield {"type": "error", "message": m_SQL_ErrorMessage}
+                                if sql.upper() not in ["EXIT", "QUIT"]:
+                                    m_SQL_ErrorMessage = "SQLCLI-0000: Script Timeout (" + \
+                                                         str(round(time.time() - self.getStartTime(), 2)) + \
+                                                         ") expired. Abort this Script."
+                                    yield {"type": "error", "message": m_SQL_ErrorMessage}
                                 raise EOFError
                             else:
                                 if m_SQLTimeOut <= 0 or \
                                         (m_ScriptTimeOut - (time.time() - self.getStartTime()) < m_SQLTimeOut):
                                     # 脚本时间已经不够，剩下的SQL时间变少
                                     m_SQLTimeOut = m_ScriptTimeOut - (time.time() - self.getStartTime())
-                        if m_SQLTimeOut > 0:
-                            executesqlFunc = func_set_timeout(timeout=m_SQLTimeOut)(executesql)
+                        if "SQL_DIRECT" in m_SQLHint.keys():
+                            self.cur.execute_direct(sql, TimeOutLimit=m_SQLTimeOut)
+                        elif "SQL_PREPARE" in m_SQLHint.keys():
+                            self.cur.execute(sql, TimeOutLimit=m_SQLTimeOut)
                         else:
-                            executesqlFunc = executesql
-                        executesqlFunc()
+                            if self.SQLOptions.get("SQL_EXECUTE").upper() == "DIRECT":
+                                self.cur.execute_direct(sql, TimeOutLimit=m_SQLTimeOut)
+                            else:
+                                self.cur.execute(sql, TimeOutLimit=m_SQLTimeOut)
                         rowcount = 0
                         m_SQL_Status = 0
                         while True:
@@ -554,19 +563,20 @@ class SQLExecute(object):
                                     }
                             if not m_FetchStatus:
                                 break
-                    except func_timeout.exceptions.FunctionTimedOut:
-                        # 从SQL语句中超时退出
-                        if self.cur is not None:
-                            m_SQL_ErrorMessage = "SQLCLI-0000: SQL timeout when execute script. aborting sql."
-                            yield {"type": "error", "message": m_SQL_ErrorMessage}
-                            self.cur.cancel()
-                            m_SQL_ErrorMessage = "SQLCLI-0000: SQL timeout when execute script. aborted sql."
-                            yield {"type": "error", "message": m_SQL_ErrorMessage}
+                    except SQLCliJDBCTimeOutException:
+                        # 处理超时时间问题
                         if m_ScriptTimeOut > 0:
                             if m_ScriptTimeOut <= time.time() - self.getStartTime():
-                                m_SQL_ErrorMessage = "SQLCLI-0000: Script timeout when execute script. abort it."
-                                yield {"type": "error", "message": m_SQL_ErrorMessage}
+                                if sql.upper() not in ["EXIT", "QUIT"]:
+                                    m_SQL_ErrorMessage = "SQLCLI-0000: Script Timeout (" + \
+                                                         str(round(time.time() - self.getStartTime(), 2)) + \
+                                                         ") expired. Abort this script."
+                                    yield {"type": "error", "message": m_SQL_ErrorMessage}
                                 raise EOFError
+                        m_SQL_ErrorMessage = "SQLCLI-0000: SQL Timeout (" + \
+                                             str(round(time.time() - self.getStartTime(), 2)) + \
+                                             ") expired. Abort this command."
+                        yield {"type": "error", "message": m_SQL_ErrorMessage}
                     except SQLCliODBCException as oe:
                         m_SQL_Status = 1
                         m_SQL_ErrorMessage = str(oe).strip()
@@ -586,14 +596,17 @@ class SQLExecute(object):
                     except (SQLCliJDBCException, Exception) as je:
                         m_SQL_Status = 1
                         m_SQL_ErrorMessage = str(je).strip()
-                        for m_ErrorPrefix in ('java.sql.SQLSyntaxErrorException:',
+                        for m_ErrorPrefix in ["java.util.concurrent.ExecutionException:", ]:
+                            if m_SQL_ErrorMessage.startswith(m_ErrorPrefix):
+                                m_SQL_ErrorMessage = m_SQL_ErrorMessage[len(m_ErrorPrefix):].strip()
+                        for m_ErrorPrefix in ['java.sql.SQLSyntaxErrorException:',
                                               "java.sql.SQLException:",
                                               "java.sql.SQLInvalidAuthorizationSpecException:",
                                               "java.sql.SQLDataException:",
                                               "java.sql.SQLTransactionRollbackException:",
                                               "java.sql.SQLTransientConnectionException:",
                                               "java.sql.SQLFeatureNotSupportedException",
-                                              "com.microsoft.sqlserver.jdbc."):
+                                              "com.microsoft.sqlserver.jdbc.", ]:
                             if m_SQL_ErrorMessage.startswith(m_ErrorPrefix):
                                 m_SQL_ErrorMessage = m_SQL_ErrorMessage[len(m_ErrorPrefix):].strip()
 
@@ -641,15 +654,6 @@ class SQLExecute(object):
                             raise SQLCliException(m_SQL_ErrorMessage)
                         else:
                             yield {"type": "error", "message": m_SQL_ErrorMessage}
-            except func_timeout.exceptions.FunctionTimedOut:
-                # 从internal命令中超时退出
-                if m_ScriptTimeOut > 0:
-                    if m_ScriptTimeOut <= time.time() - self.getStartTime():
-                        m_SQL_ErrorMessage = "SQLCLI-0000: Script timeout when execute script. abort it."
-                        yield {"type": "error", "message": m_SQL_ErrorMessage}
-                        raise EOFError
-                m_SQL_ErrorMessage = "SQLCLI-0000: SQL timeout when execute script. abort sql."
-                yield {"type": "error", "message": m_SQL_ErrorMessage}
             except EOFError:
                 # EOFError是程序退出的标志，退出应用程序
                 raise EOFError
@@ -836,6 +840,16 @@ class SQLExecute(object):
                                     m_ColumnValue = m_ColumnValue + "," + str(column[m_nPos])
                         m_ColumnValue = m_ColumnValue + "]"
                         m_row.append(m_ColumnValue)
+                    elif columntype in ("DATE", ):
+                        if isinstance(column, str):
+                            m_row.append(column)
+                        else:
+                            m_row.append(column.strftime(self.SQLOptions.get("DATE_FORMAT")))
+                    elif columntype in ("DATETIME", "TIMESTAMP"):
+                        if isinstance(column, str):
+                            m_row.append(column)
+                        else:
+                            m_row.append(column.strftime(self.SQLOptions.get("DATETIME_FORMAT")))
                     elif columntype in ("BINARY", "VARBINARY", "LONGVARBINARY"):
                         # 转换为16进制，并反算成ASCII
                         column = binascii.b2a_hex(column)
