@@ -75,6 +75,14 @@ class SQLOptions(object):
                                       "Value": "",
                                       "Comments": '',
                                       "Hidden": False})
+        self.m_SQL_OptionList.append({"Name": "DATE_FORMAT",
+                                      "Value": "%Y-%m-%d",
+                                      "Comments": '',
+                                      "Hidden": False})
+        self.m_SQL_OptionList.append({"Name": "DATETIME_FORMAT",
+                                      "Value": "%Y-%m-%d %H:%M:%S %f",
+                                      "Comments": '',
+                                      "Hidden": False})
         self.m_SQL_OptionList.append({"Name": "CONN_RETRY_TIMES",
                                       "Value": "1",
                                       "Comments": 'Connect retry times.',
